<?php if (!defined('PmWiki')) exit();
/* 
   publish page collection as book script, version 2.0.4
   Copyright 2003-2007 John Rankin john.rankin@affinity.co.nz
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This file turns a trail page into a typeset book of the contents, in a
   form suitable for printing, via the (:typeset-book:) directive.
*/

function HandleBookPublish($pagename, $auth = 'read') {
  global $HandlePublishFmt,$SpaceWikiWords,$HeadingDepth,$format;
  global $PublishFormPageFmt,$HTMLHeaderFmt,$PdfFrontMatterPattern;
  global $PrintSurTitleFmt,$PrintHeaderFmt,$PrintSubTitleFmt,$PageAnchors;
  $Tbook = array('chapter',
     'section','subsection','subsubsection','paragraph','subparagraph');
  $Chapter  = ($format=='pdf') ? 'CH:' : '-&gt;';
  $TbookDir = array('description' => 'publisher', 'keywords' => 'keywords');
  $TitleFmt = ($SpaceWikiWords) ? '$Titlespaced' : '$Title';

  $trailname = $pagename;
  $trailpage = RetrieveAuthPage($pagename, $auth, true, READPAGE_CURRENT);
  if (!$trailpage) Abort('?cannot read $pagename');
  PCache($pagename,$trailpage);
  if ($format=='pdf') {
      Markup('tbchap','block','/^CH:(.*?)(?:\s*&mdash;\s*(.*?))?$/e',
        "'<:block><tbook:aphorism>'.PSS('$1').
        ('$2' ? '<tbook:caption>'.PSS('$2').'</tbook:caption>' : '').
        '</tbook:aphorism>'");
        foreach ($TbookDir as $k => $v)
            if (preg_match("/\\(:$k\\s+(.*?):\\)/",$trailpage['text'],$dir))
                array_unshift($HTMLHeaderFmt,"<$v>$dir[1]</$v>");
      if (preg_match("/\\(:typeset-book\\s+(.*?):\\)/",$trailpage['text'],$pdf))
          SetPdfMetaTags($pdf[1]);
      SDVA($HTMLHeaderFmt,
          PublishFormDefaults($trailname,$PublishFormPageFmt['book']));
  }
  $FM = FrontMatter('default');
  foreach ($PdfFrontMatterPattern as $k => $v) SDV($$v, $FM[$k]);
  $aphorisms = ExtractChapterAphorisms($trailname,$trailpage['text']);
  $dpl = ExtractTrailStops($trailname,$trailpage['text']);
  foreach ($dpl as $k => $v) $$k = $v;

    $PublishList[] = 'markup:[[#wikipublisher.top]]';
    $counter = array(); $predepth= 0; $appendix = false;
    for($i=0;$i<count($pages);$i++) {
      @$counter[$depth[$i]-1]++;
      $num = $counter[0];
      for($j=$depth[$i];$j<$predepth;$j++) $counter[$j] = 0;
      $size = 0;
      for($j=1;$j<$depth[$i];$j++) { $num .= "." . $counter[$j]; $size++; }
      if ($size>3) $size=3;
      
      $anchor = "wikipublisher.$num";
      $num = "[[#$anchor]]";

      if ($format=='pdf') {
          $olink = TextOfLink($trailname, $links[$i]);
          $tbtag = '<'.$Tbook[$depth[$i]-1].
            (($depth[$i]==1) ? ChapterKind(strtolower($olink)) : '').'>';
          if (($depth[$i]==1) && preg_match('/^[Aa]ppendix[:\s]\s*/',$olink)) {
            $olink = preg_replace('/^[Aa]ppendix[:\s]\s*/','',$olink);
            $tbtag = ($appendix ? '' : '<appendix>').$tbtag;
            $appendix = true;
          }
          for($j=$depth[$i];$j<=$predepth;$j++) 
            $tbtag = '</'.$Tbook[$j-1].'>'.$tbtag;
          $PublishList[] = $tbtag;
          $PublishList[] = "markup:H:$num$olink";
      } else {
          $s = str_repeat('-',$size);
          $h = "!%block class='pagehead$depth[$i]'%";
          if ($size==0) $PublishList[] = "markup:$h$num$links[$i]";
          else $PublishList[] = "markup:$h"."[$s$num$links[$i]$s]";
      }      
      $predepth = $depth[$i];

      if ($pages[$i]!="textonly") {
          $PageAnchors[$pages[$i]]  = $anchor;
          $HeadingDepth[$pages[$i]] = $Tbook[$depth[$i]];
          if (isset($aphorisms[$pages[$i]])) 
              $PublishList[] = "markup:$Chapter".$aphorisms[$pages[$i]];
          if (PageExists($pages[$i])) 
	       	  $PublishList[] = "function:PrintThisPage $pages[$i]";
          else {
              $replname = TextOfLink($trailname,$links[$i]);
              $PublishList[] = "function:PrintThisText " . 
    ((MakeLink($trailname,"[[$replname]]",NULL,NULL,'$FullName')==$pages[$i]) ?
                $links[$i] : "[[".$pages[$i]."]] ($replname)");
          }
      }
    }

    if ($format=='pdf') {
        $tbtag = '';
        for($j=1;$j<=$predepth;$j++) 
            $tbtag = '</'.$Tbook[$j-1].'>'.$tbtag;
        if ($appendix) $tbtag.= '</appendix>';
        $PublishList[] = $tbtag;
    }

  $GLOBALS['PublishList'] = $PublishList;
  PrintFmt($pagename,$HandlePublishFmt);
}

function ExtractChapterAphorisms($trailname,$trailtext) {
  global $SuffixPattern; $aphorisms = array();
  foreach(explode("\n",$trailtext) as $x) {
      if (preg_match("/^:\\s*([[:upper:]\\[].*)/",$x,$match)) {
          $match[1] = ReformatTrailStop(preg_replace("/\\{(\\$.*?)\\}/e",
                "FmtPageName('$1',\$trailname)",$match[1]));
          if (preg_match("/^(\\[\\[([^|:]*?)(?:\\|([^\\[\\]]*?(?:\\[.*?\\].*?)?))?\\]\\]($SuffixPattern))\\s*:(.*)/",
          $match[1],$m)) {
              $aphorisms[MakePageName($trailname,$m[2])] = $m[5];
          }
      }
  }
  return $aphorisms;
}

function ChapterKind($kind) {
  global $ChapterKindFmt;
  $att = array('preface','introduction','acknowledgements','colophon');
  for ($j=0;$j<count($att);$j++)
    if ($kind==(isset($ChapterKindFmt[$j]) ? $ChapterKindFmt[$j] : $att[$j]))
        return " kind='$att[$j]'";
  return '';
}

?>